==> create-fly-1.21.8-6.0.8-1-sources.jar: com/zurrtum/create/content/contraptions/actors/seat/SeatBlock.java <==
package com.zurrtum.create.content.contraptions.actors.seat;

import com.google.common.base.Optional;
import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.AllEntityTags;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.api.entity.FakePlayerHandler;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import com.zurrtum.create.foundation.utility.BlockHelper;
import com.zurrtum.create.infrastructure.config.AllConfigs;
import java.util.List;
import java.util.function.Function;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1297;
import net.minecraft.class_1308;
import net.minecraft.class_1309;
import net.minecraft.class_1321;
import net.minecraft.class_1606;
import net.minecraft.class_1657;
import net.minecraft.class_1661;
import net.minecraft.class_1750;
import net.minecraft.class_1767;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_3610;
import net.minecraft.class_3726;
import net.minecraft.class_3727;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9817;

public class SeatBlock extends class_2248 implements ProperWaterloggedBlock {

    protected final class_1767 color;

    public SeatBlock(class_2251 properties, class_1767 color) {
        super(properties);
        this.color = color;
        method_9590(method_9564().method_11657(WATERLOGGED, false));
    }

    public static Function<class_2251, SeatBlock> dyed(class_1767 color) {
        return properties -> new SeatBlock(properties, color);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> pBuilder) {
        super.method_9515(pBuilder.method_11667(WATERLOGGED));
    }

    @Override
    public class_2680 method_9605(class_1750 pContext) {
        return withWater(super.method_9605(pContext), pContext);
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pDirection,
        class_2338 pNeighborPos,
        class_2680 pNeighborState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public void method_9554(class_1937 p_152426_, class_2680 p_152427_, class_2338 p_152428_, class_1297 p_152429_, double p_152430_) {
        super.method_9554(p_152426_, p_152427_, p_152428_, p_152429_, p_152430_ * 0.5F);
    }

    @Override
    public void method_9502(class_1922 reader, class_1297 entity) {
        class_2338 pos = entity.method_24515();
        if (entity instanceof class_1657 || !(entity instanceof class_1309) || !canBePickedUp(entity) || isSeatOccupied(entity.method_37908(), pos)) {
            if (entity.method_21750()) {
                super.method_9502(reader, entity);
                return;
            }

            class_243 vec3 = entity.method_18798();
            if (vec3.field_1351 < 0.0D) {
                double d0 = entity instanceof class_1309 ? 1.0D : 0.8D;
                entity.method_18800(vec3.field_1352, -vec3.field_1351 * (double) 0.66F * d0, vec3.field_1350);
            }

            return;
        }
        if (reader.method_8320(pos).method_26204() != this)
            return;
        if (entity instanceof class_9817 leashable && leashable.method_60953())
            return;
        sitDown(entity.method_37908(), pos, entity);
    }

    //TODO
    //    @Override
    //    public PathType getBlockPathType(BlockState state, BlockView world, BlockPos pos, @Nullable Mob entity) {
    //        return PathType.RAIL;
    //    }

    @Override
    public class_265 method_9530(class_2680 p_220053_1_, class_1922 p_220053_2_, class_2338 p_220053_3_, class_3726 p_220053_4_) {
        return AllShapes.SEAT;
    }

    @Override
    public class_265 method_9549(class_2680 p_220071_1_, class_1922 p_220071_2_, class_2338 p_220071_3_, class_3726 ctx) {
        if (ctx instanceof class_3727 ecc && ecc.method_32480() instanceof class_1657)
            return AllShapes.SEAT_COLLISION_PLAYERS;
        return AllShapes.SEAT_COLLISION;
    }

    public static SeatBlock getColorBlock(class_1767 color) {
        return switch (color) {
            case field_7952 -> AllBlocks.WHITE_SEAT;
            case field_7946 -> AllBlocks.ORANGE_SEAT;
            case field_7958 -> AllBlocks.MAGENTA_SEAT;
            case field_7951 -> AllBlocks.LIGHT_BLUE_SEAT;
            case field_7947 -> AllBlocks.YELLOW_SEAT;
            case field_7961 -> AllBlocks.LIME_SEAT;
            case field_7954 -> AllBlocks.PINK_SEAT;
            case field_7944 -> AllBlocks.GRAY_SEAT;
            case field_7967 -> AllBlocks.LIGHT_GRAY_SEAT;
            case field_7955 -> AllBlocks.CYAN_SEAT;
            case field_7945 -> AllBlocks.PURPLE_SEAT;
            case field_7966 -> AllBlocks.BLUE_SEAT;
            case field_7957 -> AllBlocks.BROWN_SEAT;
            case field_7942 -> AllBlocks.GREEN_SEAT;
            case field_7964 -> AllBlocks.RED_SEAT;
            case field_7963 -> AllBlocks.BLACK_SEAT;
        };
    }

    @Override
    protected class_1269 method_55765(
        class_1799 stack,
        class_2680 state,
        class_1937 level,
        class_2338 pos,
        class_1657 player,
        class_1268 hand,
        class_3965 hitResult
    ) {
        if (player.method_5715() || FakePlayerHandler.has(player))
            return class_1269.field_52423;

        class_1767 color = AllItemTags.getDyeColor(stack);
        if (color != null && color != this.color) {
            if (level.field_9236)
                return class_1269.field_5812;
            class_2680 newState = BlockHelper.copyProperties(state, getColorBlock(color).method_9564());
            level.method_8501(pos, newState);
            return class_1269.field_5812;
        }

        int x = pos.method_10263();
        int y = pos.method_10264();
        int z = pos.method_10260();
        List<SeatEntity> seats = level.method_18467(SeatEntity.class, new class_238(x, y - 0.1f, z, x + 1, y + 1, z + 1));
        if (!seats.isEmpty()) {
            SeatEntity seatEntity = seats.getFirst();
            List<class_1297> passengers = seatEntity.method_5685();
            if (!passengers.isEmpty() && passengers.getFirst() instanceof class_1657)
                return class_1269.field_52423;
            if (!level.field_9236) {
                seatEntity.method_5772();
                player.method_5804(seatEntity);
            }
            return class_1269.field_5812;
        }

        if (level.field_9236)
            return class_1269.field_5812;
        Optional<class_1297> leashed = getLeashed(level, player);
        if (leashed.isPresent()) {
            ((class_9817) leashed.get()).method_65894();
            class_1661 playerInventory = player.method_31548();
            if (!player.method_68878() || playerInventory.method_7395(stack) == -1) {
                playerInventory.method_7398(new class_1799(class_1802.field_8719));
            }
        }
        sitDown(level, pos, leashed.or(player));
        return class_1269.field_5812;
    }

    public static boolean isSeatOccupied(class_1937 world, class_2338 pos) {
        int x = pos.method_10263();
        int y = pos.method_10264();
        int z = pos.method_10260();
        return !world.method_18467(SeatEntity.class, new class_238(x, y - 0.1f, z, x + 1, y + 1, z + 1)).isEmpty();
    }

    public static Optional<class_1297> getLeashed(class_1937 level, class_1657 player) {
        List<class_1297> entities = player.method_37908().method_8333(null, player.method_5829().method_1014(10), e -> true);
        for (class_1297 e : entities)
            if (e instanceof class_1308 mob && mob.method_60952() == player && SeatBlock.canBePickedUp(e))
                return Optional.of(mob);
        return Optional.absent();
    }

    public static boolean canBePickedUp(class_1297 passenger) {
        if (passenger instanceof class_1606)
            return false;
        if (passenger instanceof class_1657)
            return false;
        if (passenger.method_5864().method_20210(AllEntityTags.IGNORE_SEAT))
            return false;
        if (!AllConfigs.server().logistics.seatHostileMobs.get() && !passenger.method_5864().method_5891().method_6136())
            return false;
        return passenger instanceof class_1309;
    }

    public static void sitDown(class_1937 world, class_2338 pos, class_1297 entity) {
        if (world.field_9236)
            return;
        SeatEntity seat = new SeatEntity(world, pos);
        seat.method_5814(pos.method_10263() + .5, pos.method_10264(), pos.method_10260() + .5);
        world.method_8649(seat);
        entity.method_5873(seat, true);
        if (entity instanceof class_1321 ta)
            ta.method_6179(true);
    }

    public class_1767 getColor() {
        return color;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

}

==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/contraptions/actors/seat/SeatBlock.java <==
package com.zurrtum.create.content.contraptions.actors.seat;

import com.google.common.base.Optional;
import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.AllEntityTags;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.api.entity.FakePlayerHandler;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import com.zurrtum.create.foundation.utility.BlockHelper;
import com.zurrtum.create.infrastructure.config.AllConfigs;
import java.util.List;
import java.util.function.Function;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1297;
import net.minecraft.class_1308;
import net.minecraft.class_1309;
import net.minecraft.class_1321;
import net.minecraft.class_1606;
import net.minecraft.class_1657;
import net.minecraft.class_1661;
import net.minecraft.class_1750;
import net.minecraft.class_1767;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_3610;
import net.minecraft.class_3726;
import net.minecraft.class_3727;
import net.minecraft.class_3965;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9817;

public class SeatBlock extends class_2248 implements ProperWaterloggedBlock {

    protected final class_1767 color;

    public SeatBlock(class_2251 properties, class_1767 color) {
        super(properties);
        this.color = color;
        method_9590(method_9564().method_11657(WATERLOGGED, false));
    }

    public static Function<class_2251, SeatBlock> dyed(class_1767 color) {
        return properties -> new SeatBlock(properties, color);
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> pBuilder) {
        super.method_9515(pBuilder.method_11667(WATERLOGGED));
    }

    @Override
    public class_2680 method_9605(class_1750 pContext) {
        return withWater(super.method_9605(pContext), pContext);
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pDirection,
        class_2338 pNeighborPos,
        class_2680 pNeighborState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public void method_9554(class_1937 p_152426_, class_2680 p_152427_, class_2338 p_152428_, class_1297 p_152429_, double p_152430_) {
        super.method_9554(p_152426_, p_152427_, p_152428_, p_152429_, p_152430_ * 0.5F);
    }

    @Override
    public void method_9502(class_1922 reader, class_1297 entity) {
        class_2338 pos = entity.method_24515();
        if (entity instanceof class_1657 || !(entity instanceof class_1309) || !canBePickedUp(entity) || isSeatOccupied(entity.method_37908(), pos)) {
            if (entity.method_21750()) {
                super.method_9502(reader, entity);
                return;
            }

            class_243 vec3 = entity.method_18798();
            if (vec3.field_1351 < 0.0D) {
                double d0 = entity instanceof class_1309 ? 1.0D : 0.8D;
                entity.method_18800(vec3.field_1352, -vec3.field_1351 * (double) 0.66F * d0, vec3.field_1350);
            }

            return;
        }
        if (reader.method_8320(pos).method_26204() != this)
            return;
        if (entity instanceof class_9817 leashable && leashable.method_60953())
            return;
        sitDown(entity.method_37908(), pos, entity);
    }

    //TODO
    //    @Override
    //    public PathType getBlockPathType(BlockState state, BlockView world, BlockPos pos, @Nullable Mob entity) {
    //        return PathType.RAIL;
    //    }

    @Override
    public class_265 method_9530(class_2680 p_220053_1_, class_1922 p_220053_2_, class_2338 p_220053_3_, class_3726 p_220053_4_) {
        return AllShapes.SEAT;
    }

    @Override
    public class_265 method_9549(class_2680 p_220071_1_, class_1922 p_220071_2_, class_2338 p_220071_3_, class_3726 ctx) {
        if (ctx instanceof class_3727 ecc && ecc.method_32480() instanceof class_1657)
            return AllShapes.SEAT_COLLISION_PLAYERS;
        return AllShapes.SEAT_COLLISION;
    }

    public static SeatBlock getColorBlock(class_1767 color) {
        return switch (color) {
            case field_7952 -> AllBlocks.WHITE_SEAT;
            case field_7946 -> AllBlocks.ORANGE_SEAT;
            case field_7958 -> AllBlocks.MAGENTA_SEAT;
            case field_7951 -> AllBlocks.LIGHT_BLUE_SEAT;
            case field_7947 -> AllBlocks.YELLOW_SEAT;
            case field_7961 -> AllBlocks.LIME_SEAT;
            case field_7954 -> AllBlocks.PINK_SEAT;
            case field_7944 -> AllBlocks.GRAY_SEAT;
            case field_7967 -> AllBlocks.LIGHT_GRAY_SEAT;
            case field_7955 -> AllBlocks.CYAN_SEAT;
            case field_7945 -> AllBlocks.PURPLE_SEAT;
            case field_7966 -> AllBlocks.BLUE_SEAT;
            case field_7957 -> AllBlocks.BROWN_SEAT;
            case field_7942 -> AllBlocks.GREEN_SEAT;
            case field_7964 -> AllBlocks.RED_SEAT;
            case field_7963 -> AllBlocks.BLACK_SEAT;
        };
    }

    @Override
    protected class_1269 method_55765(
        class_1799 stack,
        class_2680 state,
        class_1937 level,
        class_2338 pos,
        class_1657 player,
        class_1268 hand,
        class_3965 hitResult
    ) {
        if (player.method_5715() || FakePlayerHandler.has(player))
            return class_1269.field_52423;

        class_1767 color = AllItemTags.getDyeColor(stack);
        if (color != null && color != this.color) {
            if (level.field_9236)
                return class_1269.field_5812;
            class_2680 newState = BlockHelper.copyProperties(state, getColorBlock(color).method_9564());
            level.method_8501(pos, newState);
            return class_1269.field_5812;
        }

        List<SeatEntity> seats = level.method_18467(SeatEntity.class, new class_238(pos));
        if (!seats.isEmpty()) {
            SeatEntity seatEntity = seats.getFirst();
            List<class_1297> passengers = seatEntity.method_5685();
            if (!passengers.isEmpty() && passengers.getFirst() instanceof class_1657)
                return class_1269.field_52423;
            if (!level.field_9236) {
                seatEntity.method_5772();
                player.method_5804(seatEntity);
            }
            return class_1269.field_5812;
        }

        if (level.field_9236)
            return class_1269.field_5812;
        Optional<class_1297> leashed = getLeashed(level, player);
        if (leashed.isPresent()) {
            ((class_9817) leashed.get()).method_65894();
            class_1661 playerInventory = player.method_31548();
            if (!player.method_68878() || playerInventory.method_7395(stack) == -1) {
                playerInventory.method_7398(new class_1799(class_1802.field_8719));
            }
        }
        sitDown(level, pos, leashed.or(player));
        return class_1269.field_5812;
    }

    public static boolean isSeatOccupied(class_1937 world, class_2338 pos) {
        return !world.method_18467(SeatEntity.class, new class_238(pos)).isEmpty();
    }

    public static Optional<class_1297> getLeashed(class_1937 level, class_1657 player) {
        List<class_1297> entities = player.method_37908().method_8333(null, player.method_5829().method_1014(10), e -> true);
        for (class_1297 e : entities)
            if (e instanceof class_1308 mob && mob.method_60952() == player && SeatBlock.canBePickedUp(e))
                return Optional.of(mob);
        return Optional.absent();
    }

    public static boolean canBePickedUp(class_1297 passenger) {
        if (passenger instanceof class_1606)
            return false;
        if (passenger instanceof class_1657)
            return false;
        if (passenger.method_5864().method_20210(AllEntityTags.IGNORE_SEAT))
            return false;
        if (!AllConfigs.server().logistics.seatHostileMobs.get() && !passenger.method_5864().method_5891().method_6136())
            return false;
        return passenger instanceof class_1309;
    }

    public static void sitDown(class_1937 world, class_2338 pos, class_1297 entity) {
        if (world.field_9236)
            return;
        SeatEntity seat = new SeatEntity(world, pos);
        seat.method_5814(pos.method_10263() + .5, pos.method_10264(), pos.method_10260() + .5);
        world.method_8649(seat);
        entity.method_5873(seat, true);
        if (entity instanceof class_1321 ta)
            ta.method_6179(true);
    }

    public class_1767 getColor() {
        return color;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

}
